/*
*  Name:      TInteger.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type integer.
*  Date:      $Date: 2002/10/04 11:01:56 $
*  Revision:  $Revision: 1.7 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;


///  SQL type for integer numbers.
public class TInteger implements IType
{

  private Long   tValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TInteger() {}

  /**
  *  Builds a new instance.
  *  @param kiVALUE Source integer.
  *  @post ( !isNull() )
  */
  public TInteger (final long kiVALUE)
  {
    tValue = new Long (kiVALUE);
  }

  /**
  *  Builds a new instance.
  *  @param kyINTEGER Source integer.
  *  @post ( isNull() == ( null == kyINTEGER ) )
  */
  public TInteger (final String kyINTEGER)
  {
    if ( kyINTEGER != null )
    {
      tValue = new Long (kyINTEGER);
    }
  }

  public IType set (final Object ktVALUE)
  {

    if ( ktVALUE == null )
    {
      tValue = null;
    }
    else if ( ktVALUE instanceof Integer )
    {
      tValue = new Long (((Integer) ktVALUE).longValue());
    }
    else if ( ktVALUE instanceof Long )
    {
      tValue = new Long (((Long) ktVALUE).longValue());
    }
    else if ( ktVALUE instanceof String )
    {
      tValue = new Long ((String) ktVALUE);
    }
    else
    {
      throw new IllegalArgumentException ("invalid instance of " + ktVALUE.getClass());
    }
    return this;

  }  // set()


  //
  //  S E L E C T O R S
  //

  public String get()
  {

    String   yResult = "";

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      yResult = tValue.toString();
    }
    return yResult;

  }  // get()

  public int intValue()
  {
    return tValue.intValue();
  }

  public boolean isNull()
  {
    return ( tValue == null );
  }

  public long longValue()
  {
    return tValue.longValue();
  }

  public String toString()
  {
    return tValue.toString();
  }

}  // class TInteger
