/*
*  Name:      TReal.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type real.
*  Date:      $Date: 2002/09/28 18:54:48 $
*  Revision:  $Revision: 1.6 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;

import java.math.BigDecimal;


///  SQL type for real numbers.
public class TReal implements IType
{

  private Double   tValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TReal() {}

  /**
  *  Builds a new instance.
  *  @param kfVALUE Source float number.
  *  @post ( !isNull() )
  */
  public TReal (final float kfVALUE)
  {
    tValue = new Double (kfVALUE);
  }

  /**
  *  Builds a new instance.
  *  @param kdVALUE Source double number.
  *  @post ( isNull() == ( null == kfVALUE ) )
  */
  public TReal (final double kdVALUE)
  {
    tValue = new Double (kdVALUE);
  }

  /**
  *  Builds a new instance.
  *  @param kyREAL Source real.
  *  @post ( isNull() == ( null == kyREAL ) )
  */
  public TReal (final String kyREAL)
  {
    if ( kyREAL != null )
    {
      tValue = new Double (kyREAL);
    }
  }

  public IType set (final Object ktVALUE)
  {

    if ( ktVALUE == null )
    {
      tValue = null;
    }
    else if ( ktVALUE instanceof BigDecimal )
    {
      tValue = new Double (((BigDecimal) ktVALUE).doubleValue());
    }
    else if ( ktVALUE instanceof Double )
    {
      tValue = new Double (((Double) ktVALUE).doubleValue());
    }
    else if ( ktVALUE instanceof Float )
    {
      tValue = new Double (((Float) ktVALUE).doubleValue());
    }
    else if ( ktVALUE instanceof String )
    {
      tValue = new Double ((String) ktVALUE);
    }
    else
    {
      throw new IllegalArgumentException ("invalid instance of " + ktVALUE.getClass());
    }
    return this;

  }  // set()


  //
  //  S E L E C T O R S
  //

  public double doubleValue()
  {
    return tValue.doubleValue();
  }

  public float floatValue()
  {
    return tValue.floatValue();
  }

  public String get()
  {

    String   yResult;

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      yResult = tValue.toString();
    }
    return yResult;

  }  // get()

  public boolean isNull()
  {
    return ( tValue == null );
  }

  public String toString()
  {
    return tValue.toString();
  }

}  // class TReal
