#include <mpcl/log.hh>
#include "odbc-3.x.hh"

#line 1 "test-1.uesql.cc"
/*
*  Name:         test-1.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2002/10/16 22:44:49 $
*  Revision:     $Revision: 1.17 $
*
*  Copyright (C) 1996-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestCursors (void)
{

  using std::cerr;
  using std::endl;
  using uesql::TIdentifier;
  using uesql::TInteger;
  using uesql::TReal;
  using uesql::TSearchCondition;
  using uesql::TString;
  
  TString       yVar0;
  TString       yVar1;
  TReal         tHours;
  TInteger      tZero (0L);
  bool          gEmpty       = true;
  TIdentifier   yAttribName  = "work_code";
  int           iRecordCount = 0;

  TEST_INIT ("test for cursors");

#line 84 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 85 "test-1.uesql.cc"

#line 86 "test-1.uesql.cc"

#line 90 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
mpcl::TString                    hyCursorName;

uesql::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesql::htOdbc3xManager.execute ("SELECT NAME, HOURS FROM WORKERS WHERE ( NAME = 'nobody' );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesql::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 91 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();


}  // not ODBC 3.x
#line 92 "test-1.uesql.cc"

  while ( true )
  {
#line 96 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 97 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;
int             hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar1);

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesql::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 98 "test-1.uesql.cc"

    gEmpty = false;
  }

loop_end:
#line 104 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 105 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;

uesql::htOdbc3xManager.clearQueues();

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

uesql::htOdbc3xManager.clearResult (hrtResultTriple);
uesql::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 106 "test-1.uesql.cc"

  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "WORK_CODE NAME                                       HOURS\n");
    fprintf (hptTestTargetFile, "========= ========================================== =====\n");
  }
#line 122 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 126 "test-1.uesql.cc"

#line 127 "test-1.uesql.cc"

#line 132 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
mpcl::TString                    hyCursorName;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (yAttribName);

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesql::htOdbc3xManager.execute ("SELECT $z, NAME, HOURS FROM WORKERS;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesql::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 133 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();


#line 134 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;
int             hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (tZero);
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar1);
uesql::htOdbc3xManager.pushOnOutputQueue (&tHours);

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetchScroll (hrtResultTriple.tSqlhstmt, SQL_FETCH_ABSOLUTE, std::atoi (uesql::htOdbc3xManager.instantiate ("$z").c_str()));
uesql::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end_2;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 135 "test-1.uesql.cc"

  TEST_FAIL;
  goto next;

loop_end_2:
  TEST_PASS;

next:  
  while ( true )
  {
#line 146 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 147 "test-1.uesql.cc"

#line 148 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;
int             hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar1);
uesql::htOdbc3xManager.pushOnOutputQueue (&tHours);

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesql::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end_3;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 149 "test-1.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5.1f\n", yVar0.get().c_str(), yVar1.get().c_str(), (double) tHours);
    }
    ++iRecordCount;
  }

loop_end_3:
#line 159 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 163 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;

uesql::htOdbc3xManager.clearQueues();

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

uesql::htOdbc3xManager.clearResult (hrtResultTriple);
uesql::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 164 "test-1.uesql.cc"

  if ( iRecordCount > 0 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // _TestCursors()


static int _TestSqlTypes (void)
{

  TEST_INIT ("test for SQL types");

  using std::cerr;
  using std::endl;
  using std::size_t;
  using uesql::TDate;
  using uesql::TInteger;
  using uesql::TReal;
  using uesql::TString;
  using uesql::TTime;

  TDate      tDate;
  TInteger   tInteger;
  TReal      tReal;
  TTime      tTime;
  TString    yString;
  int        iRecordCount = 0;

  TEST_NUMBERS (1,     TInteger ().set ("1"));
  TEST_NUMBERS (0,     TInteger ().set ("0"));
  TEST_NUMBERS (10000, TInteger ().set ("10000"));
  TEST_NUMBERS (10001, TInteger (10001));
  TEST_NUMBERS (true,  TDate    ().set ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TInteger ().set ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TReal    ().set ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TString  ().set ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TDate    ().set ("").isNull());
  TEST_NUMBERS (true,  TInteger ().set ("").isNull());
  TEST_NUMBERS (true,  TReal    ().set ("").isNull());
  TEST_NUMBERS (false, TString  ().set ("").isNull());
  TEST_NUMBERS (true,  TDate    ().isNull());
  TEST_NUMBERS (true,  TInteger ().isNull());
  TEST_NUMBERS (true,  TReal    ().isNull());
  TEST_NUMBERS (true,  TString  ().isNull());
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.set ("75000");
  TEST_NUMBERS (75000, tInteger);
#line 231 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 232 "test-1.uesql.cc"

#line 233 "test-1.uesql.cc"

#line 241 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("CREATE TABLE TABLE_1 (CODE CHARACTER (3) NOT NULL, BIRTH_DATE DATE, BIRTH_TIME TIME, SALARY INTEGER, PERCENTAGE DOUBLE PRECISION);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 242 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('000', '1999-12-31', '01:00', 12000, 25.75);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 243 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('001', '1970-01-01', '02:00', 75000, 30);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 244 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('002', '2000-01-01', '03:00', 11500, 99.99);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 249 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (TDate (2000, 12, 31));
uesql::htOdbc3xManager.pushOnInputQueue (TTime ("12:00:00"));
uesql::htOdbc3xManager.pushOnInputQueue (TInteger (7500));
uesql::htOdbc3xManager.pushOnInputQueue (TReal (33.33));

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('003', $z, $z, $z, $z);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 254 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (TDate (2001, 1, 1));
uesql::htOdbc3xManager.pushOnInputQueue (TTime (23, 59, 59));
uesql::htOdbc3xManager.pushOnInputQueue (TInteger (6500));
uesql::htOdbc3xManager.pushOnInputQueue (TReal (0.0));

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('004', $z, $z, $z, $z);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 256 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
mpcl::TString                    hyCursorName;

uesql::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesql::htOdbc3xManager.execute ("SELECT CODE, BIRTH_DATE, BIRTH_TIME, SALARY, PERCENTAGE FROM TABLE_1;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesql::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 257 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();


}  // not ODBC 3.x
#line 258 "test-1.uesql.cc"


  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE  BIRTH_DATE   BIRTH_DATE SALARY PERCENTAGE");
    fprintf (hptTestTargetFile, "%s\n", "===== ============ ========== ====== ==========");
  }
  while ( true )
  {
#line 271 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 272 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;
int             hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&yString);
uesql::htOdbc3xManager.pushOnOutputQueue (&tDate);
uesql::htOdbc3xManager.pushOnOutputQueue (&tTime);
uesql::htOdbc3xManager.pushOnOutputQueue (&tInteger);
uesql::htOdbc3xManager.pushOnOutputQueue (&tReal);

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesql::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 273 "test-1.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf ( hptTestTargetFile              ,
                "%5s %12s %10s %6ld %10.03f\n" ,
                yString.get().c_str()          ,
                tDate.get().c_str()            ,
                tTime.get().c_str()            ,
                (long int) tInteger            ,
                (double) tReal                 );
    }
    ++iRecordCount;
  }

loop_end:
#line 289 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 290 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;

uesql::htOdbc3xManager.clearQueues();

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

uesql::htOdbc3xManager.clearResult (hrtResultTriple);
uesql::htOdbc3xManager.removeCursor (hyCursorName);
}
}
#line 291 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DROP TABLE TABLE_1 CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 292 "test-1.uesql.cc"

  if ( iRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSqlTypes()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TIdentifier;
  using uesql::TInteger;
  using uesql::TSearchCondition;
  using uesql::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  TString*           pyDbmsUserLogin  = &yDbmsUserLogin;

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 390 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 394 "test-1.uesql.cc"

#line 395 "test-1.uesql.cc"

#line 400 "test-1.uesql.cc"
{
mpcl::TString              hySqlSchemaLocation;
mpcl::TString              hyDatabaseName;
mpcl::TString              hyConnectionName;
mpcl::TString              hyUserLogin;
mpcl::TString::size_type   zSolidusPos;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesql::htOdbc3xManager.pushOnInputQueue (yDbmsUserLogin);

//
//  '$z' has been omitted.
//
hySqlSchemaLocation = uesql::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = "DEFAULT";
hyUserLogin         = uesql::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesql::htOdbc3xManager.isConnected (hyConnectionName) )
{
  uesql::htOdbc3xManager.unbindConnection (hyConnectionName);
}
if ( uesql::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesql::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 409 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("CREATE TABLE JOBS (CODE CHARACTER (3) NOT NULL, WORKER_COUNT INTEGER);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 416 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("CREATE TABLE WORKERS (WORK_CODE CHARACTER (3) NOT NULL, NAME CHARACTER (40) NOT NULL, HOURS DECIMAL (5));");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 421 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("ALTER TABLE WORKERS ADD COLUMN STATE CHARACTER (40);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 422 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("ALTER TABLE WORKERS ADD COLUMN PHONE_NUMBER INTEGER;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 427 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (yCodeValue);

htResultTriple = uesql::htOdbc3xManager.execute ("DELETE FROM JOBS WHERE ( CODE = $z );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 428 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DELETE FROM JOBS WHERE ( CODE = '001' );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 429 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DELETE FROM WORKERS;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 434 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO JOBS VALUES ('001', 100);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 435 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO JOBS VALUES ('002', 250);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 436 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 437 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 438 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 439 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 440 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 441 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'william', NULL, 'Missouri', 5550005);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 442 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'peter', NULL, 'Arkansas', 5550006);");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 443 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 444 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 449 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&tCondition);

uesql::htOdbc3xManager.popFromOutputQueue()->set (uesql::htOdbc3xManager.instantiate ("( WORK_CODE = CODE )"));
#line 454 "test-1.uesql.cc"

#line 461 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar1);
uesql::htOdbc3xManager.pushOnInputQueue (tCondition);

htResultTriple = uesql::htOdbc3xManager.execute ("SELECT HOURS FROM WORKERS, JOBS WHERE $z;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto not_found;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto cardinality_check;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesql::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesql::SQLCODE    = -1;
    uesql::SQLMESSAGE = "";
    uesql::SQLSTATE   = "21000";
    goto cardinality_check;
  }
}
}
}  // not ODBC 3.x
#line 462 "test-1.uesql.cc"

    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesql::SQLSTATE.c_str(), uesql::cardinality_violation::pkcNO_SUBCLASS);
#line 468 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 472 "test-1.uesql.cc"

#line 477 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&tCount);

htResultTriple = uesql::htOdbc3xManager.execute ("SELECT COUNT (*) FROM WORKERS WHERE ( HOURS IS NULL );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto not_found;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesql::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesql::SQLCODE    = -1;
    uesql::SQLMESSAGE = "";
    uesql::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 482 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
mpcl::TString                    hyCursorName;

uesql::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesql::htOdbc3xManager.execute ("SELECT WORK_CODE, HOURS FROM WORKERS WHERE ( HOURS IS NULL );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesql::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 483 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();


#line 484 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;
int             hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesql::htOdbc3xManager.pushOnOutputQueue (&tHours);

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesql::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  goto not_found;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htOdbc3xManager.updateOutputVariables();
}
}
}
#line 485 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;

uesql::htOdbc3xManager.clearQueues();

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

uesql::htOdbc3xManager.clearResult (hrtResultTriple);
uesql::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 486 "test-1.uesql.cc"

    TEST_NUMBERS (tCount,          4);
    TEST_NUMBERS (tCount.isNull(), false);
    TEST_NUMBERS (yVar0.isNull(),  false);
    TEST_NUMBERS (tHours.isNull(), true);
    
    yCursorName = "mycursor";
#line 494 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 498 "test-1.uesql.cc"

#line 503 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&tCount);

htResultTriple = uesql::htOdbc3xManager.execute ("SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto not_found;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesql::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesql::SQLCODE    = -1;
    uesql::SQLMESSAGE = "";
    uesql::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 508 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&tCondition);

uesql::htOdbc3xManager.popFromOutputQueue()->set (uesql::htOdbc3xManager.instantiate ("( WORKERS.NAME = 'myname' )"));
#line 509 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnOutputQueue (&tCondition);
uesql::htOdbc3xManager.pushOnInputQueue (tCondition);

uesql::htOdbc3xManager.popFromOutputQueue()->set (uesql::htOdbc3xManager.instantiate ("( $z AND ( WORKERS.NAME = NAME ) )"));
#line 516 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (tCondition);

htResultTriple = uesql::htOdbc3xManager.execute ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE $z;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 517 "test-1.uesql.cc"

    TEST_FUNCTION (_TestCursors());
    TEST_FUNCTION (_TestSqlTypes());
#line 521 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 525 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;
mpcl::TString                    hyCursorName;

uesql::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesql::htOdbc3xManager.execute ("SELECT CODE FROM JOBS WHERE ( CODE = '001' );");
uesql::htOdbc3xManager.diagnose (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  uesql::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesql::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 526 "test-1.uesql.cc"
uesql::htOdbc3xManager.clearQueues();


#line 527 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;
int             hI = 0;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (yCursorName);
uesql::htOdbc3xManager.pushOnOutputQueue (&yVar0);

hyCursorName = uesql::htOdbc3xManager.instantiate ("$z");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesql::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesql::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesql::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesql::htOdbc3xManager.isNotFoundCondition() )
{
  goto not_found;
}
else if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htOdbc3xManager.updateOutputVariables();
}
}
}
#line 528 "test-1.uesql.cc"
{
mpcl::TString   hyCursorName;

uesql::htOdbc3xManager.clearQueues();

hyCursorName = uesql::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesql::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "34000";
  goto error;
}
else
{
uesql::odbc_3_x::TResultTriple&   hrtResultTriple (uesql::htOdbc3xManager.getCursor (hyCursorName));

uesql::htOdbc3xManager.clearResult (hrtResultTriple);
uesql::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 529 "test-1.uesql.cc"

    TEST_STRINGS (uesql::SQLSTATE.c_str(), uesql::successful_completion::pkcNO_SUBCLASS);
    TEST_NUMBERS (tCount,        4);
    TEST_STRINGS (yVar0.c_str(), "001");
#line 534 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 538 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("REVOKE ALL PRIVILEGES ON JOBS FROM PUBLIC CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 539 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("REVOKE ALL PRIVILEGES ON WORKERS FROM PUBLIC RESTRICT;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 544 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("GRANT SELECT ON JOBS TO PUBLIC;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 545 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("GRANT SELECT ON WORKERS TO PUBLIC;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 546 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("GRANT DELETE ON WORKERS TO PUBLIC;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 551 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 556 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DROP VIEW VIEW1 CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 561 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DROP TABLE JOBS CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 566 "test-1.uesql.cc"
{
uesql::htOdbc3xManager.clearQueues();

if ( uesql::htOdbc3xManager.existsDefaultConnection() )
{
  uesql::htOdbc3xManager.resetStatus();
  uesql::htOdbc3xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08003";
  goto error;
}
}
}  // not ODBC 3.x
#line 567 "test-1.uesql.cc"


#line 570 "test-1.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 574 "test-1.uesql.cc"
{
mpcl::TString              hySqlSchemaLocation;
mpcl::TString              hyDatabaseName;
mpcl::TString              hyConnectionName;
mpcl::TString              hyUserLogin;
mpcl::TString::size_type   zSolidusPos;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesql::htOdbc3xManager.pushOnInputQueue (*pyDbmsUserLogin);

hySqlSchemaLocation = uesql::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = uesql::htOdbc3xManager.instantiate ("_'myconnection'");
hyUserLogin         = uesql::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
hyConnectionName.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesql::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesql::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 579 "test-1.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DROP TABLE WORKERS CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 580 "test-1.uesql.cc"

    goto end;

  error:
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
