/*
*  Name:      TTest1.uesql.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2002/10/16 22:44:49 $
*  Revision:  $Revision: 1.18 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

import java.text.DecimalFormat;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.uesqlc.TDate;
import org.uesqlc.TIdentifier;
import org.uesqlc.TInteger;
import org.uesqlc.TNotFoundException;
import org.uesqlc.TReal;
import org.uesqlc.TSearchCondition;
import org.uesqlc.TSqlerrorException;
import org.uesqlc.TState;
import org.uesqlc.TString;
import org.uesqlc.TTime;


///  Class for testing UESQL support in JAVA.
class TTest1
{

  ///  True if instance must not print informational messages.
  private boolean   gSilentMode = false;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Sets silent mode status.
  *  @param gSILENT_MODE Silent mode.
  */
  void setSilentMode (boolean gSILENT_MODE)
  {
    gSilentMode = gSILENT_MODE;
  }

  /**
  *  Test SQL cursors.
  *  @return Exit code.
  */
  int testCursors()
  {

    boolean       gEmpty       = true;
    int           iExitCode    = 0;
    int           iRecordCount = 0;
    TReal         tHours       = new TReal (0);
    TInteger      tZero        = new TInteger (0);
    TIdentifier   yAttribName  = new TIdentifier ("work_code");
    TString       yVar0        = new TString();
    TString       yVar1        = new TString();

//   TEST_INIT ("test for cursors");

    try
    {
      extern "UESQL::default"
      {
        whenever sqlerror  throw TSqlerrorException;
        whenever not found throw TNotFoundException;
        declare mycursor cursor for
          select name, hours
            from workers
            where ( name = 'nobody' );
        open mycursor;
      }
      try
      {
        while ( true )
        {
          extern "UESQL::default"
          {
            fetch from mycursor into :yVar0, :yVar1;
          }
          gEmpty = false;
        }
      }
      catch (final TNotFoundException ktEXCEPTION)
      {
        extern "UESQL::default" 
        {
          close mycursor;
        }
        assert ( gEmpty );
      }

      try
      {
        extern "UESQL::default"
        {
          --
          --  DECLARE CURSOR/OPEN/WHENEVER
          --
          declare mycursor cursor for select :yAttribName, name, hours from workers;
          open mycursor;
          fetch absolute :tZero from mycursor into :yVar0, :yVar1, :tHours;
        }
        assert false;
      }
      catch (final TNotFoundException ktINNER_EXCEPTION) {}

      try
      {
        if ( !gSilentMode )
        {
          System.err.println ("WORK_CODE NAME                                       HOURS");
          System.err.println ("========= ========================================== =====");
        }
        while ( true )
        {
          extern "UESQL::default"
          {
            fetch from mycursor into :yVar0, :yVar1, :tHours;
          }
          if ( !gSilentMode )
          {
            System.err.print ("    " + yVar0.get());
            System.err.print (" "    + yVar1.get());
            System.err.print ("  "   + tHours.get());
            System.err.println();
          }
          ++iRecordCount;
        }
      }
      catch (final TNotFoundException ktEXCEPTION)
      {
        extern "UESQL::default"
        {
          --
          --  CLOSE CURSOR
          --
          close mycursor;
        }
      }
    }
    catch (final TSqlerrorException ktEXCEPTION)
    {
      System.err.println ("SQLSTATE="    + TState.SQLSTATE);
      System.err.println ("SQLCODE="     + TState.SQLCODE);
      System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
      iExitCode = TState.SQLCODE;
    }
    return iExitCode;

  }  // testCursors()

  /**
  *  Test SQL types.
  *  @return Exit code.
  */
  int testSqlTypes()
  {

//     TEST_INIT ("test for SQL types");

    TDate           tDate1;
    TDate           tDate2;
    TInteger        tInteger1;
    TInteger        tInteger2;
    TReal           tReal1;
    TReal           tReal2;
    TTime           tTime1;
    TTime           tTime2;
    int             iExitCode      = 0;
    int             iRecordCount   = 0;
    TDate           tDate          = new TDate();
    TInteger        tInteger       = new TInteger();
    TReal           tReal          = new TReal();
    TTime           tTime          = new TTime();
    TString         yString        = new TString();
    DecimalFormat   tIntegerFormat = new DecimalFormat (" ######");
    DecimalFormat   tPercentFormat = new DecimalFormat (" ######.000");

    tIntegerFormat.setMinimumIntegerDigits (6);
    tPercentFormat.setMinimumIntegerDigits (6);
    assert (     1 == new TInteger ("1").intValue() );
    assert (     0 == new TInteger ("0").intValue() );
    assert ( 10000 == new TInteger ("10000").intValue() );
    assert ( 10001 == new TInteger (10001).intValue() );
    assert ( new TDate    ((String) null).isNull() );
    assert ( new TInteger (null).isNull() );
    assert ( new TReal    (null).isNull() );
    assert ( new TString  (null).isNull() );
    assert ( new TDate    ("").isNull() );
    assert ( new TInteger ("").isNull() );
    assert ( new TReal    ("").isNull() );
    assert ( new TString  ("").isNull() );
    assert ( new TDate    ().isNull() );
    assert ( new TInteger ().isNull() );
    assert ( new TReal    ().isNull() );
    assert ( new TString  ().isNull() );
    tInteger.set ("75000");
    assert ( 75000 == tInteger.intValue() );

    tDate1    = new TDate (2000, 12, 31);
    tDate2    = new TDate (2001,  1,  1);
    tInteger1 = new TInteger (7500);
    tInteger2 = new TInteger (6500);
    tReal1    = new TReal (33.33);
    tReal2    = new TReal (0.0);
    tTime1    = new TTime ("12:00:00");
    tTime2    = new TTime (23, 59, 59);
    try
    {
      extern "UESQL::default" 
      {
        whenever sqlerror  throw TSqlerrorException;
        whenever not found throw TNotFoundException;
        create table table_1
        (
          code       character (3) not null ,
          birth_date date                   ,
          birth_time time                   ,
          salary     integer                ,
          percentage double precision
        );
        insert into table_1 values ('000', '1999-12-31', '01:00', 12000,      25.75);
        insert into table_1 values ('001', '1970-01-01', '02:00', 75000,      30);
        insert into table_1 values ('002', '2000-01-01', '03:00', 11500,      99.99);
        insert into table_1 values ('003', :tDate1,      :tTime1, :tInteger1, :tReal1);
        insert into table_1 values ('004', :tDate2,      :tTime2, :tInteger2, :tReal2);
        declare mycursor cursor for
          select code, birth_date, birth_time, salary, percentage from table_1;
        open mycursor;
      }
    }
    catch (final TSqlerrorException ktEXCEPTION)
    {
      System.err.println ("SQLSTATE="    + TState.SQLSTATE);
      System.err.println ("SQLCODE="     + TState.SQLCODE);
      System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
      iExitCode = -1;
    }

    if ( !gSilentMode )
    {
      System.err.println ("CODE  BIRTH_DATE   BIRTH_TIME SALARY PERCENTAGE");
      System.err.println ("===== ============ ========== ====== ==========");
    }
    try
    {
      while ( true )
      {
        extern "UESQL::default"
        {
          fetch from mycursor into :yString, :tDate, :tTime, :tInteger, :tReal;
        }
        if ( !gSilentMode )
        {
          System.err.print (yString.get());
          System.err.print (" " + tDate.get());
          System.err.print (" " + tTime.get());
          System.err.print (tIntegerFormat.format (tInteger.intValue()));
          System.err.print (tPercentFormat.format (tReal.doubleValue()));
          System.err.println();
        }
        ++iRecordCount;
      }
    }
    catch (final TSqlerrorException ktEXCEPTION)
    {
      System.err.println ("SQLSTATE="    + TState.SQLSTATE);
      System.err.println ("SQLCODE="     + TState.SQLCODE);
      System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
      iExitCode = -1;
    }
    catch (final TNotFoundException ktEXCEPTION)
    {
      if ( iRecordCount != 5 )
      {
        System.err.println ("SQLSTATE="    + TState.SQLSTATE);
        System.err.println ("SQLCODE="     + TState.SQLCODE);
        System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
        iExitCode = TState.SQLCODE;
      }
    }
    assert ( iRecordCount == 5 );

    try
    {
      extern "UESQL::default" 
      {
        close mycursor;
        drop table table_1 cascade;
      }
    }
    catch (final TSqlerrorException ktEXCEPTION)
    {
      System.err.println ("SQLSTATE="    + TState.SQLSTATE);
      System.err.println ("SQLCODE="     + TState.SQLCODE);
      System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
      iExitCode = -1;
    }
    return iExitCode;

//     TEST_MEMORY_STATUS;
//     TEST_RETURN_CODE;

  }  // testSqlTypes()

  /**
  *  Entry point.  It uses the user login 'test/test'  and the schema  location
  *  'localhost/test'. It reads the next system properties:
  *  org.uesqlc.dbms_server_name DBMS server name.
  *  org.uesqlc.test_mode        AUTOMAKE test mode ('silent_mode', 'interactive_mode').
  *  @param ayARGS Array of string arguments.
  *  @see ../../../README.
  */
  public static void main (String ayARGS[])
  {

    boolean            gSilentMode;
    TIdentifier        yCursorName;
    TString            yDbmsUserLogin;
    TString            yDbmsSchemaLocation;
    int                iExitCode       = 0;
    TInteger           tCount          = new TInteger();
    TInteger           tHours          = new TInteger();
    TSearchCondition   tCondition      = new TSearchCondition();
    TTest1             tTest1          = new TTest1();
    TString            yCodeValue      = new TString ("002");
    String             yDbmsServerName = "localhost";
    TString            yVar0           = new TString();
    TString            yVar1           = new TString();
    TString            yVar2           = new TString();

    //
    //  Activates logging depending on system property 'org.uesqlc.testmode'.
    //
    yDbmsServerName = System.getProperty ("org.uesqlc.dbms_server_name", yDbmsServerName);
    gSilentMode     =
      System.getProperty ("org.uesqlc.test_mode", "interactive_mode").equals ("silent_mode");
    if ( gSilentMode )
    {
      Logger.getLogger ("org.uesqlc").setLevel (Level.OFF);
    }
    else
    {
      Logger.getLogger ("org.uesqlc").setLevel (Level.ALL);
    }
    if ( !gSilentMode )
    {
      System.err.println ("DBMS-SCHEMA-LOCATION = " + TString._composeSchemaLocation (yDbmsServerName, "test"));
      System.err.println ("DBMS-USER-LOGIN      = " + TString._composeUserLogin ("test", "test"));
    }
    try
    {
      extern "UESQL::default" 
      {
        --
        --  WHENEVER
        --
        whenever sqlerror throw TSqlerrorException;

        --
        --  CONNECT TO
        --
        connect
          to   :TString._composeSchemaLocation (yDbmsServerName, "test")
          user :TString._composeUserLogin ("test", "test");

        --
        --  CREATE TABLE
        --
        create table jobs
        (
          code         character (3) not null ,
          worker_count integer
        );

        create table workers
        (
          work_code character (3)  not null ,
          name      character (40) not null ,
          hours     decimal (5)
        );

        --
        --  ALTER TABLE
        --
        alter table workers add column state character (40);
        alter table workers add column phone_number integer;

        --
        --  DELETE
        --
        delete from jobs where ( code = :yCodeValue );
        delete from jobs where ( code = '001' );
        delete from workers;

        --
        --  INSERT
        --
        insert into jobs values ('001', 100);
        insert into jobs values ('002', 250);
        insert into workers values ('001', 'myname',   40,   'Andalucia',   5550000);
        insert into workers values ('001', 'yourname', 35,   'Extremadura', 5550001);
        insert into workers values ('001', 'hisname',  25,   'Catalunya',   5550002);
        insert into workers values ('002', 'john',     35,   'New England', 5550003);
        insert into workers values ('002', 'james',    35,   'Washington',  5550004);
        insert into workers values ('002', 'william',  null, 'Missouri',    5550005);
        insert into workers values ('002', 'peter',    null, 'Arkansas',    5550006);
        insert into workers select work_code, name, hours, state, phone_number from workers where ( name = 'myname' );
        insert into workers select work_code, name, hours, state, phone_number from workers;

        --
        --  DECLARE CONDITION
        --
        declare :tCondition condition where ( work_code = code );
      }

      //
      //  Checks for cardinality violation.
      //
      try
      {
        extern "UESQL::default"
        {
          --
          --  WHENEVER
          --
          whenever sqlerror throw TSqlerrorException;

          --
          --  SELECT for a single row.
          --
          --    Cardinality violation, so ( SQLSTATE == '21000' ).
          --
          select hours into :tHours from workers, jobs where :tCondition;
        }
        iExitCode = -1;
      }
      catch (final TSqlerrorException ktEXCEPTION)
      {
        if ( TState.SQLSTATE == TState.cardinality_violation._kyNO_SUBCLASS )
        {
          iExitCode = 0;
        }
        else
        {
          System.err.println ("SQLSTATE="    + TState.SQLSTATE);
          System.err.println ("SQLCODE="     + TState.SQLCODE);
          System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
          iExitCode = -1;
        }
      }
      catch (final TNotFoundException ktEXCEPTION)
      {
        System.err.println ("SQLSTATE="    + TState.SQLSTATE);
        System.err.println ("SQLCODE="     + TState.SQLCODE);
        System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
        iExitCode = TState.SQLCODE;
      }

      extern "UESQL::default" 
      {
        --
        --  WHENEVER
        --
        whenever sqlerror throw TSqlerrorException;

        --
        --  SELECT for a single row.
        --
        select count (*) into :tCount from workers where ( hours is null );

        --
        --  DECLARE CURSOR/OPEN/FETCH/CLOSE
        --
        declare MYCURSOR cursor for select work_code, hours from workers where ( hours is null );
        open MYCURSOR;
        fetch from MYCURSOR into :yVar0, :tHours;
        close MYCURSOR;
      }
      assert ( tCount.intValue() == 4 );
      assert ( !tCount.isNull() );
      assert ( !yVar0.isNull() );
      assert ( tHours.isNull() );
    
      yCursorName = new TIdentifier ("mycursor");
      extern "UESQL::default" 
      {
        --
        --  WHENEVER
        --
        whenever sqlerror throw TSqlerrorException;

        --
        --  SELECT for a single row.
        --
        select count (*) into :tCount from workers where name = 'myname';

        --
        --  DECLARE CONDITION
        --
        declare :tCondition condition where ( workers.name = 'myname' );
        declare :tCondition condition where ( :tCondition and ( workers.name = name ) );

        --
        --  UPDATE
        --
        update workers
          set phone_number = workers.phone_number
          where :tCondition;
      }
      tTest1.setSilentMode (gSilentMode);
      tTest1.testCursors();
      tTest1.testSqlTypes();
      extern "UESQL::default" 
      {
        --
        --  DECLARE CURSOR/OPEN/FETCH/CLOSE
        --
        declare mycursor cursor for select code from jobs where code = '001';
        open mycursor;
        fetch from :yCursorName into :yVar0;
        close mycursor;
      }
      assert ( TState.SQLSTATE  == TState.successful_completion._kyNO_SUBCLASS );
      assert ( yVar2.toString() == "4" );
      assert ( yVar0.toString() == "001");

      extern "UESQL::default" 
      {
        --
        --  REVOKE
        --
        revoke all privileges on jobs    from public cascade;
        revoke all privileges on workers from public restrict;

        --
        --  GRANT
        --
        grant select on jobs    to public;
        grant select on workers to public;
        grant delete on workers to public;

        --
        --  CREATE VIEW
        --
        create view view1 as select * from jobs;

        --
        --  DROP VIEW
        --
        drop view view1 cascade;

        --
        --  DROP TABLE
        --
        drop table jobs cascade;

        --
        --  DISCONNECT
        --
        disconnect current;
      }

      extern "UESQL::default"
      {
        --
        --  CONNECT TO
        --
        connect
          to   :TString._composeSchemaLocation (yDbmsServerName, "test")
          as   'myconnection'
          user :TString._composeUserLogin ("test", "test");

        --
        --  DROP TABLE
        --
        drop table workers cascade;
      }
    }
    catch (final TSqlerrorException ktEXCEPTION)
    {
      System.err.println ("SQLSTATE="    + TState.SQLSTATE);
      System.err.println ("SQLCODE="     + TState.SQLCODE);
      System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
      iExitCode = TState.SQLCODE;
    }
    catch (final TNotFoundException ktEXCEPTION)
    {
      System.err.println ("SQLSTATE="    + TState.SQLSTATE);
      System.err.println ("SQLCODE="     + TState.SQLCODE);
      System.err.println ("DESCRIPTION=" + TState.SQLMESSAGE);
      iExitCode = TState.SQLCODE;
    }
    System.exit (iExitCode);

  }  // main()

}  // class TTest1
