/*
*  Name:      test-5.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2002/09/29 21:23:06 $
*  Revision:  $Revision: 1.7 $
*
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/log.hh>
#include <mpcl/operating_system.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TString;
  
  TEST_INIT ("Test for SCHEMA related statements");
  TEST_START_WATCH;

  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TString            yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition goto label 'error'.
      --
      whenever sqlerror goto error;

      --
      --  Connects to default schema.
      --
      connect to :yDbmsServerName user :yDbmsUserLogin;

      --
      --  Creates schema 'test_5_schema'.
      --
      create schema test_5_schema
        create table table_1
        (
          code         character (3) not null ,
          worker_count integer
        )
        create table table_2
        (
          work_code character (3)  not null ,
          name      character (40) not null ,
          hours     decimal   (5)
        );

      --
      --  Connects to the newly created schema.
      --
      connect to :ComposeSchemaLocation (yDbmsServerName, "test_5_schema")
        as 'myconnection'
        user :yDbmsUserLogin;

      --
      --  Removes tables.
      --
      drop table table_1 cascade;
      disconnect current;
      set connection default;
    }

    //
    //  Sleeps for 5  seconds to  let slow  systems (like POSTGRESQL on a i586)
    //  remove connections properly.
    //
    mpcl::tOperatingSystem.sleep (5);
    extern "UESQL::default" 
    {
      --
      --  Removes schema 'test_5_schema'.
      --
      drop schema test_5_schema cascade;
    }
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
